@@ -58,6 +58,8 @@ extern "C" {            			/* Use "C" linkage when in C++ mode */
 #define s32 int32_t
 #define s64 int64_t
 
+#define FP_SP_SIZE 4
+#define FP_DP_SIZE 8
 
 /*
  * General EAX, EBX, ECX, EDX type registers.  Note that for
@@ -76,6 +78,10 @@ extern "C" {            			/* Use "C" linkage when in C++ mode */
 
 #ifdef	__BIG_ENDIAN__
 
+typedef struct {
+  u8 reg[16];
+} I128_reg_t;
+
 typedef struct {
   u32 e_reg;
 } I32_reg_t;
@@ -90,6 +96,10 @@ typedef struct {
 
 #else /* !__BIG_ENDIAN__ */
 
+typedef struct {
+  u8 reg[16];
+} I128_reg_t;
+
 typedef struct {
   u32 e_reg;
 } I32_reg_t;
@@ -119,6 +129,9 @@ struct i386_special_regs {
   u32 FLAGS;
 };
 
+struct i386_see_regs {
+  I128_reg_t XMM[8];
+};
 
 typedef struct {
   union {
@@ -216,6 +229,16 @@ typedef struct {
 #define R_EIP		spc.IP.I32_reg.e_reg
 #define R_EFLG		spc.FLAGS
 
+/* SSE registers */
+#define R_XMM0		sse.XMM[0]
+#define R_XMM1		sse.XMM[1]
+#define R_XMM2		sse.XMM[2]
+#define R_XMM3		sse.XMM[3]
+#define R_XMM4		sse.XMM[4]
+#define R_XMM5		sse.XMM[5]
+#define R_XMM6		sse.XMM[6]
+#define R_XMM7		sse.XMM[7]
+
 /* segment registers */
 #define R_ES_INDEX	0
 #define R_CS_INDEX	1
@@ -304,15 +327,16 @@ typedef struct {
 #define R_REAL_TSC	msr[0x12]
 
 /* flag conditions   */
-#define FB_CF 0x0001            /* CARRY flag  */
-#define FB_PF 0x0004            /* PARITY flag */
-#define FB_AF 0x0010            /* AUX  flag   */
-#define FB_ZF 0x0040            /* ZERO flag   */
-#define FB_SF 0x0080            /* SIGN flag   */
-#define FB_TF 0x0100            /* TRAP flag   */
-#define FB_IF 0x0200            /* INTERRUPT ENABLE flag */
-#define FB_DF 0x0400            /* DIR flag    */
-#define FB_OF 0x0800            /* OVERFLOW flag */
+#define FB_CF 0x000001            /* CARRY flag  */
+#define FB_PF 0x000004            /* PARITY flag */
+#define FB_AF 0x000010            /* AUX  flag   */
+#define FB_ZF 0x000040            /* ZERO flag   */
+#define FB_SF 0x000080            /* SIGN flag   */
+#define FB_TF 0x000100            /* TRAP flag   */
+#define FB_IF 0x000200            /* INTERRUPT ENABLE flag */
+#define FB_DF 0x000400            /* DIR flag    */
+#define FB_OF 0x000800            /* OVERFLOW flag */
+#define FB_ID 0x200000            /* ID flag */
 
 /* 80286 and above always have bit#1 set */
 #define F_ALWAYS_ON  (0x0002)   /* flag bits always on */
@@ -335,6 +359,11 @@ typedef struct {
 #define F_DF 0x0400             /* DIR flag    */
 #define F_OF 0x0800             /* OVERFLOW flag */
 
+#define CR0_MP		(1 <<  1)
+#define CR0_EM		(1 <<  2)
+#define CR4_OSFXSR	(1 <<  9)
+#define CR4_OSXMMEXCPT	(1 << 10)
+
 #define X86EMU_TOGGLE_FLAG(emu, flag)     (emu->x86.R_FLG ^= (flag))
 #define X86EMU_SET_FLAG(emu, flag)        (emu->x86.R_FLG |= (flag))
 #define X86EMU_CLEAR_FLAG(emu, flag)      (emu->x86.R_FLG &= ~(flag))
@@ -380,11 +409,15 @@ struct x86emu_s;
 typedef unsigned (* x86emu_memio_handler_t)(struct x86emu_s *, u32 addr, u32 *val, unsigned type);
 typedef int (* x86emu_intr_handler_t)(struct x86emu_s *, u8 num, unsigned type);
 typedef int (* x86emu_code_handler_t)(struct x86emu_s *);
+typedef void (* x86emu_cpuid_handler_t)(struct x86emu_s *);
+typedef void (* x86emu_wrmsr_handler_t)(struct x86emu_s *);
+typedef void (* x86emu_rdmsr_handler_t)(struct x86emu_s *);
 typedef void (* x86emu_flush_func_t)(struct x86emu_s *, char *buf, unsigned size);
 
 typedef struct {
   struct i386_general_regs gen;
   struct i386_special_regs spc;
+  struct i386_see_regs sse;
   sel_t seg[8];
   sel_t ldt;
   sel_t tr;
@@ -486,8 +519,11 @@ x86			- X86 registers
 typedef struct x86emu_s {
   x86emu_regs_t x86;
   x86emu_code_handler_t code_check;
+  x86emu_cpuid_handler_t cpuid;
   x86emu_memio_handler_t memio;
   x86emu_intr_handler_t intr;
+  x86emu_wrmsr_handler_t wrmsr;
+  x86emu_wrmsr_handler_t rdmsr;
   x86emu_mem_t *mem;
   struct {
     unsigned char *map;
@@ -532,6 +568,9 @@ void x86emu_set_io_perm(x86emu_t *emu, unsigned start, unsigned end, unsigned pe
 void x86emu_set_page(x86emu_t *emu, unsigned page, void *address);
 void x86emu_reset_access_stats(x86emu_t *emu);
 
+x86emu_rdmsr_handler_t x86emu_set_rdmsr_handler(x86emu_t *emu, x86emu_rdmsr_handler_t handler);
+x86emu_wrmsr_handler_t x86emu_set_wrmsr_handler(x86emu_t *emu, x86emu_wrmsr_handler_t handler);
+x86emu_cpuid_handler_t x86emu_set_cpuid_handler(x86emu_t *emu, x86emu_cpuid_handler_t handler);
 x86emu_code_handler_t x86emu_set_code_handler(x86emu_t *emu, x86emu_code_handler_t handler);
 x86emu_intr_handler_t x86emu_set_intr_handler(x86emu_t *emu, x86emu_intr_handler_t handler);
 x86emu_memio_handler_t x86emu_set_memio_handler(x86emu_t *emu, x86emu_memio_handler_t handler);
