@@ -56,12 +56,12 @@ static void vm_w_word(x86emu_mem_t *vm, unsigned addr, unsigned val);
 static void vm_w_dword(x86emu_mem_t *vm, unsigned addr, unsigned val);
 
 static mem2_page_t *vm_get_page(x86emu_mem_t *mem, unsigned addr, int create);
-static unsigned vm_i_byte(unsigned addr);
-static unsigned vm_i_dword(unsigned addr);
-static unsigned vm_i_word(unsigned addr);
-static void vm_o_byte(unsigned addr, unsigned val);
-static void vm_o_dword(unsigned addr, unsigned val);
-static void vm_o_word(unsigned addr, unsigned val);
+static unsigned vm_i_byte(x86emu_t *emu, unsigned addr);
+static unsigned vm_i_dword(x86emu_t *emu, unsigned addr);
+static unsigned vm_i_word(x86emu_t *emu, unsigned addr);
+static void vm_o_byte(x86emu_t *emu, unsigned addr, unsigned val);
+static void vm_o_dword(x86emu_t *emu, unsigned addr, unsigned val);
+static void vm_o_word(x86emu_t *emu, unsigned addr, unsigned val);
 
 void *mem_dup(const void *src, size_t n)
 {
@@ -148,7 +148,7 @@ x86emu_mem_t *emu_mem_clone(x86emu_mem_t *mem)
 }
 
 
-void x86emu_reset_access_stats(x86emu_t *emu)
+API_SYM void x86emu_reset_access_stats(x86emu_t *emu)
 {
   mem2_pdir_t *pdir;
   mem2_ptable_t *ptable;
@@ -172,7 +172,7 @@ void x86emu_reset_access_stats(x86emu_t *emu)
 }
 
 
-void x86emu_set_io_perm(x86emu_t *emu, unsigned start, unsigned end, unsigned perm)
+API_SYM void x86emu_set_io_perm(x86emu_t *emu, unsigned start, unsigned end, unsigned perm)
 {
   if(!emu) return;
 
@@ -236,7 +236,7 @@ mem2_page_t *vm_get_page(x86emu_mem_t *mem, unsigned addr, int create)
 }
 
 
-void x86emu_set_perm(x86emu_t *emu, unsigned start, unsigned end, unsigned perm)
+API_SYM void x86emu_set_perm(x86emu_t *emu, unsigned start, unsigned end, unsigned perm)
 {
   x86emu_mem_t *mem;
   mem2_page_t *page;
@@ -284,7 +284,7 @@ void x86emu_set_perm(x86emu_t *emu, unsigned start, unsigned end, unsigned perm)
 }
 
 
-void x86emu_set_page(x86emu_t *emu, unsigned page, void *address)
+API_SYM void x86emu_set_page(x86emu_t *emu, unsigned page, void *address)
 {
   x86emu_mem_t *mem;
   mem2_page_t *p;
@@ -563,20 +563,20 @@ void vm_w_dword(x86emu_mem_t *mem, unsigned addr, unsigned val)
 }
 
 
-unsigned vm_i_byte(unsigned addr)
+unsigned vm_i_byte(x86emu_t *emu, unsigned addr)
 {
   unsigned char *perm;
 
   addr &= 0xffff;
-  perm = M.io.map + addr;
+  perm = emu->io.map + addr;
 
   if(
-    M.io.iopl_ok &&
+    emu->io.iopl_ok &&
     (*perm & X86EMU_PERM_R)
   ) {
     *perm |= X86EMU_ACC_R;
 
-    M.io.stats_i[addr]++;
+    emu->io.stats_i[addr]++;
 
     return inb(addr);
   }
@@ -584,28 +584,28 @@ unsigned vm_i_byte(unsigned addr)
     *perm |= X86EMU_ACC_INVALID;
   }
 
-  M.mem->invalid = 1;
+  emu->mem->invalid = 1;
 
   return 0xff;
 }
 
 
-unsigned vm_i_word(unsigned addr)
+unsigned vm_i_word(x86emu_t *emu, unsigned addr)
 {
   unsigned char *perm;
   unsigned val;
 
   addr &= 0xffff;
-  perm = M.io.map + addr;
+  perm = emu->io.map + addr;
 
   if(
-    !M.io.iopl_ok ||
+    !emu->io.iopl_ok ||
     addr == 0xffff ||
     !(perm[0] & X86EMU_PERM_R) ||
     !(perm[1] & X86EMU_PERM_R)
   ) {
-    val = vm_i_byte(addr);
-    val += (vm_i_byte(addr + 1) << 8);
+    val = vm_i_byte(emu, addr);
+    val += (vm_i_byte(emu, addr + 1) << 8);
 
     return val;
   }
@@ -613,33 +613,33 @@ unsigned vm_i_word(unsigned addr)
   perm[0] |= X86EMU_ACC_R;
   perm[1] |= X86EMU_ACC_R;
 
-  M.io.stats_i[addr]++;
-  M.io.stats_i[addr + 1]++;
+  emu->io.stats_i[addr]++;
+  emu->io.stats_i[addr + 1]++;
 
   return inw(addr);
 }
 
 
-unsigned vm_i_dword(unsigned addr)
+unsigned vm_i_dword(x86emu_t *emu, unsigned addr)
 {
   unsigned char *perm;
   unsigned val;
 
   addr &= 0xffff;
-  perm = M.io.map + addr;
+  perm = emu->io.map + addr;
 
   if(
-    !M.io.iopl_ok ||
+    !emu->io.iopl_ok ||
     addr >= 0xfffd ||
     !(perm[0] & X86EMU_PERM_R) ||
     !(perm[1] & X86EMU_PERM_R) ||
     !(perm[2] & X86EMU_PERM_R) ||
     !(perm[3] & X86EMU_PERM_R)
   ) {
-    val = vm_i_byte(addr);
-    val += (vm_i_byte(addr + 1) << 8);
-    val += (vm_i_byte(addr + 2) << 16);
-    val += (vm_i_byte(addr + 3) << 24);
+    val = vm_i_byte(emu, addr);
+    val += (vm_i_byte(emu, addr + 1) << 8);
+    val += (vm_i_byte(emu, addr + 2) << 16);
+    val += (vm_i_byte(emu, addr + 3) << 24);
 
     return val;
   }
@@ -649,55 +649,55 @@ unsigned vm_i_dword(unsigned addr)
   perm[2] |= X86EMU_ACC_R;
   perm[3] |= X86EMU_ACC_R;
 
-  M.io.stats_i[addr]++;
-  M.io.stats_i[addr + 1]++;
-  M.io.stats_i[addr + 2]++;
-  M.io.stats_i[addr + 3]++;
+  emu->io.stats_i[addr]++;
+  emu->io.stats_i[addr + 1]++;
+  emu->io.stats_i[addr + 2]++;
+  emu->io.stats_i[addr + 3]++;
 
   return inl(addr);
 }
 
 
-void vm_o_byte(unsigned addr, unsigned val)
+void vm_o_byte(x86emu_t *emu, unsigned addr, unsigned val)
 {
   unsigned char *perm;
 
   addr &= 0xffff;
-  perm = M.io.map + addr;
+  perm = emu->io.map + addr;
 
   if(
-    M.io.iopl_ok &&
+    emu->io.iopl_ok &&
     (*perm & X86EMU_PERM_W)
   ) {
     *perm |= X86EMU_ACC_W;
 
-    M.io.stats_o[addr]++;
+    emu->io.stats_o[addr]++;
 
     outb(val, addr);
   }
   else {
     *perm |= X86EMU_ACC_INVALID;
 
-    M.mem->invalid = 1;
+    emu->mem->invalid = 1;
   }
 }
 
 
-void vm_o_word(unsigned addr, unsigned val)
+void vm_o_word(x86emu_t *emu, unsigned addr, unsigned val)
 {
   unsigned char *perm;
 
   addr &= 0xffff;
-  perm = M.io.map + addr;
+  perm = emu->io.map + addr;
 
   if(
-    !M.io.iopl_ok ||
+    !emu->io.iopl_ok ||
     addr == 0xffff ||
     !(perm[0] & X86EMU_PERM_W) ||
     !(perm[1] & X86EMU_PERM_W)
   ) {
-    vm_o_byte(addr, val);
-    vm_o_byte(addr + 1, val);
+    vm_o_byte(emu, addr, val);
+    vm_o_byte(emu, addr + 1, val);
 
     return;
   }
@@ -705,32 +705,32 @@ void vm_o_word(unsigned addr, unsigned val)
   perm[0] |= X86EMU_ACC_W;
   perm[1] |= X86EMU_ACC_W;
 
-  M.io.stats_o[addr]++;
-  M.io.stats_o[addr + 1]++;
+  emu->io.stats_o[addr]++;
+  emu->io.stats_o[addr + 1]++;
 
   outw(val, addr);
 }
 
 
-void vm_o_dword(unsigned addr, unsigned val)
+void vm_o_dword(x86emu_t *emu, unsigned addr, unsigned val)
 {
   unsigned char *perm;
 
   addr &= 0xffff;
-  perm = M.io.map + addr;
+  perm = emu->io.map + addr;
 
   if(
-    !M.io.iopl_ok ||
+    !emu->io.iopl_ok ||
     addr >= 0xfffd ||
     !(perm[0] & X86EMU_PERM_W) ||
     !(perm[1] & X86EMU_PERM_W) ||
     !(perm[2] & X86EMU_PERM_W) ||
     !(perm[3] & X86EMU_PERM_W)
   ) {
-    vm_o_byte(addr, val);
-    vm_o_byte(addr + 1, val);
-    vm_o_byte(addr + 2, val);
-    vm_o_byte(addr + 3, val);
+    vm_o_byte(emu, addr, val);
+    vm_o_byte(emu, addr + 1, val);
+    vm_o_byte(emu, addr + 2, val);
+    vm_o_byte(emu, addr + 3, val);
 
     return;
   }
@@ -740,10 +740,10 @@ void vm_o_dword(unsigned addr, unsigned val)
   perm[2] |= X86EMU_ACC_W;
   perm[3] |= X86EMU_ACC_W;
 
-  M.io.stats_o[addr]++;
-  M.io.stats_o[addr + 1]++;
-  M.io.stats_o[addr + 2]++;
-  M.io.stats_o[addr + 3]++;
+  emu->io.stats_o[addr]++;
+  emu->io.stats_o[addr + 1]++;
+  emu->io.stats_o[addr + 2]++;
+  emu->io.stats_o[addr + 3]++;
 
   outl(val, addr);
 }
@@ -810,13 +810,13 @@ unsigned vm_memio(x86emu_t *emu, u32 addr, u32 *val, unsigned type)
     case X86EMU_MEMIO_I:
       switch(bits) {
         case X86EMU_MEMIO_8:
-          *val = vm_i_byte(addr);
+          *val = vm_i_byte(emu, addr);
           break;
         case X86EMU_MEMIO_16:
-          *val = vm_i_word(addr);
+          *val = vm_i_word(emu, addr);
           break;
         case X86EMU_MEMIO_32:
-          *val = vm_i_dword(addr);
+          *val = vm_i_dword(emu, addr);
           break;
       }
       break;
@@ -824,13 +824,13 @@ unsigned vm_memio(x86emu_t *emu, u32 addr, u32 *val, unsigned type)
     case X86EMU_MEMIO_O:
       switch(bits) {
         case X86EMU_MEMIO_8:
-          vm_o_byte(addr, *val);
+          vm_o_byte(emu, addr, *val);
           break;
         case X86EMU_MEMIO_16:
-          vm_o_word(addr, *val);
+          vm_o_word(emu, addr, *val);
           break;
         case X86EMU_MEMIO_32:
-          vm_o_dword(addr, *val);
+          vm_o_dword(emu, addr, *val);
           break;
       }
       break;
